<?php

class Card {

    private $dbc = null;

    private $card_token = null;
    
    private $card_id = null;

    function __construct($dbc) {
    
        $this->dbc = $dbc;

    }

    function load_card($card_token) {

        $sql = "SELECT s.*, c.*, meta.* 
                from card c 
                    INNER JOIN lorcana_meta meta ON c.id=meta.card_id
                    INNER JOIN `set` s ON s.id = c.`set_id`
                WHERE c.token = '".Common::sanitize_filter_string($card_token, [FILTER_FLAG_NO_ENCODE_QUOTES])."'";

        $card_data = $this->dbc->select($sql, true);

        // TODO - Duplicates of the same card
        if (count($card_data) == 2) {
            $card_data = $card_data[0]; 
        }

        if ($card_data == []) {
            header('Location: /');
            exit;
        }

        $data = pathinfo($card_data['image_normal']);
        $parts = explode('?', $data['basename']);

        $card_data['image_normal'] = '/img/cards/lorcana/normal_'.$parts[0];
        
        if ($card_data['card_type'] != 'Character') {
            $card_data['card_type'] = str_replace(',', ' &#9702; ', $card_data['card_type']);
        }

        $this->card_id = $card_data['id'];

        return $card_data;

    }

    function get_inventory() {

        $inventory_data = [];

        $sql = "SELECT * FROM inventory WHERE card_id=".$this->card_id.";";

        $results = $this->dbc->select($sql);

        foreach($results as $inventory) {
            $type = $inventory['printing'] == 0 ? 'normal' : 'foil';
            $inventory['quality'] = Common::card_quality($inventory['quality']);
            $inventory_data[$type] = $inventory;
        }

        return $inventory_data;

    }

    function get_card_listings() {

        $listings = [];

        $listings = [
            [
                'id' => '87264',
                'name' => 'Jill',
                'rating' => '99% (14 sales)',
                'quantiry' => 2,
                'price' => 4.45,
                'postage' => 1.22,
            ],
            [
                'id' => '87264',
                'name' => 'Bob',
                'rating' => '98% (62 sales)',
                'quantiry' => 1,
                'price' => 4.50,
                'postage' => 1.22,
            ],
            [
                'id' => '82473',
                'name' => 'Rach',
                'rating' => '95% (55 sales)',
                'quantiry' => 5,
                'price' => 4.55,
                'postage' => 1.22,
            ],
            [
                'id' => '98234',
                'name' => 'Sally',
                'rating' => '100% (25 sales)',
                'quantiry' => 3,
                'price' => 4.55,
                'postage' => 1.22,
            ],
            [
                'id' => '13267',
                'name' => 'Nick',
                'rating' => '75% (43 sales)',
                'quantiry' => 7,
                'price' => 4.65,
                'postage' => 1.22,
            ],
        ];

        return $listings;

    }

    public static function generate_token($name, $version='', $rarity='Common') {

        $token = '';

        $name = preg_replace( '/[^a-z\- ]/i', '', strtolower($name));
        $token = str_replace(' ', '-', $name);

        if ($version) {
            $version = preg_replace( '/[^a-z\- ]/i', '', strtolower($version));
            $token .= '-' . str_replace(' ', '-', strtolower($version));
        }

        if ($rarity == 'Enchanted') {
            $token .= '-enchanted';
        }
        
        return $token;

    }

    function get_market_data() {

        $sql = "SELECT FROM_UNIXTIME(mp.timestamp, '%Y %m %d') as `date`, mp.`normal_trend`, mp.`foil_trend`
                FROM `card` c 
                    INNER JOIN `market_price` mp ON mp.cardmarket_id=c.cardmarket_id
                WHERE c.id = ".$this->card_id.";";

        $result_data = $this->dbc->select($sql, true);

        $market_data = [];

        foreach($result_data as $data) {

            $market_data[$data['date']] = $data;

        }

        return $market_data;

    }

    function get_trend_direction($market_data) {

        $normal_card_price_data = [];
        $foil_card_price_data = [];
        
        foreach($market_data as $data) {
            $normal_card_price_data[] = $data['normal_trend'];
            $foil_card_price_data[] = $data['foil_trend'];
        }
        
        // debug($normal_card_price_data, false);
        // debug($foil_card_price_data, false);

        $normal_x = range(1, count($normal_card_price_data));
        $normal_fit = Common::linear_regression($normal_x, $normal_card_price_data);

        $normal_trend = 'flat';
        if ($normal_fit['beta'] > 0.01) {
            $normal_trend = 'up';
        } else if ($normal_fit['beta'] < -0.01) {
            $normal_trend = 'down';
        } 

        $foil_x = range(1, count($foil_card_price_data));
        $foil_fit = Common::linear_regression($foil_x, $foil_card_price_data);

        $foil_trend = 'flat';
        if ($foil_fit['beta'] > 0.01) {
            $foil_trend = 'up';
        } else if ($foil_fit['beta'] < -0.01) {
            $foil_trend = 'down';
        } 

        // debug($normal_fit, false);
        // debug($foil_fit, false);
        // debug($normal_trend, false);
        // debug($foil_trend);

		return [
            'normal' => $normal_trend,
            'foil' => $foil_trend,
        ];

    }

}