<?php

class Common {

	static function highlight_value() {

		$values = [
			['token' => 'fun', 'title' => 'We play for fun!', 'description' => 'Playing our favourite game among many others things brings us joy.' ],
			['token' => 'care', 'title' => 'We care for our friends', 'description' => 'Spending quality time together enriches friendships.' ],
			['token' => 'satisfaction', 'title' => 'We are collectors at heart', 'description' => 'There is great satisfaction in completing our collection.' ],
			['token' => 'thinkers', 'title' => 'We love a good strategy', 'description' => 'Crafting our strategies and seeing them work is an awesome feeling!' ],
			['token' => 'excitement', 'title' => 'We celebrate together', 'description' => 'We are thrilled finding the cards we have been chasing!' ],
		];

		return $values[rand(0, count($values)-1)];

	}

	static function check_plain($text) {

		$text = str_replace('..', '', $text);

		return htmlspecialchars($text, ENT_QUOTES, 'UTF-8');

	}

	static function sanitize_number($string) {

		return preg_replace("/[^0-9]/", "", $string);

	}

	static function sanitize_filter_string($value, array $flags = [FILTER_FLAG_NO_ENCODE_QUOTES]): string
	{
		$noQuotes = in_array(FILTER_FLAG_NO_ENCODE_QUOTES, $flags);
		$options = ($noQuotes ? ENT_NOQUOTES : ENT_QUOTES) | ENT_SUBSTITUTE;
		$optionsDecode = ($noQuotes ? ENT_QUOTES : ENT_NOQUOTES) | ENT_SUBSTITUTE;

		// Strip the tags
		$value = strip_tags($value);

		// Run the replacement for FILTER_SANITIZE_STRING
		$value = htmlspecialchars($value, $options);

		// Fix that HTML entities are converted to entity numbers instead of entity name (e.g. ' -> &#34; and not ' -> &quote;)
		// https://stackoverflow.com/questions/64083440/use-php-htmlentities-to-convert-special-characters-to-their-entity-number-rather
		$value = str_replace(["&quot;", "&#039;"], ["&#34;", "&#39;"], $value);

		// Decode all entities
		return html_entity_decode($value, $optionsDecode);
	}

	public static function card_quality($value) {

        if ($value == 4) {
            return "Mint / Near Mint";
        } else if ($value == 3) {
            return "Slightly Played";
        } else if ($value == 2) {
            return "Played";
        } else if ($value == 1) {
            return "Heavily Played";
        } else if ($value == 0) {
            return "Poor";
        }

        return 'Unknown';
    }

	public static function linear_regression($x, $y)
	{
		$x_sum = array_sum($x);
		$y_sum = array_sum($y);
		$xy_sum = 0;
		$x2_sum = 0;
		$n = count($x);
		for($i=0;$i<$n;$i++)
		{
			$xy_sum += $x[$i] * $y[$i];
			$x2_sum += $x[$i] * $x[$i];
		}
		$beta = ($n * $xy_sum - $x_sum * $y_sum) / ($n * $x2_sum - $x_sum * $x_sum);
		$alpha = $y_sum / $n - $beta * $x_sum / $n;
		return ['alpha' => $alpha, 'beta' => $beta];
	}
}
