<?php

class LorcanaBrowse {

    private $dbc = null;

    public static $set_list = [
        '1TFC',
        '2ROF',
        '3INK',
        '4URS',
        '5SSK',
        '6AZS',
    ];
    public static $card_type_list = [
        'Character',
        'Action',
        'Item',
        'Song',
    ];
    public static $card_rarity_list = [
        'Common',
        'Uncommon',
        'Rare',
        'Super Rare',
        'Legendary',
        'Enchanted'
    ];
    public static $card_printing_list = [
        'Normal' => 0,
        'Cold Foil' => 1,
    ];
    public static $order_sort_list = [
        'a-z',
        'low-to-high',
        'high-to-low',
        'best-selling',
    ];

    function __construct($dbc) {
        
        $this->dbc = $dbc;
        
        //debug($_GET, false);

    }

    function search() {

        $tokens = LorcanaBrowse::sanitise_tokens();

        $sql = "SELECT c.id, c.`name`, c.`version`, c.token, s.name as set_name, MATCH(c.`name`,c.`version`) AGAINST ('".implode(' ', $tokens)."' IN BOOLEAN MODE) as score
                FROM card c 
                    INNER JOIN `set` s ON s.id = c.`set_id`
                    INNER JOIN `inventory` i ON i.`card_id` = c.`id`
                WHERE MATCH(c.`name`,c.`version`) AGAINST ('".implode(' ', $tokens)."' IN BOOLEAN MODE) > 0
                ORDER BY score DESC
                LIMIT 10";

        $result_data = $this->dbc->select($sql);

        $search_results = [];

        foreach($result_data as $card) {
            $search_results[] = [
                'card_id' => $card['id'],
                'card_name' => isset($card['version']) ? $card['name'] . ' - ' . $card['version'] : $card['name'],
                'card_token' => $card['token'],
                'card_set' => $card['set_name'],
            ];
        }

        return $search_results;

    }

    public static function user_filters() {

        $where_condition = [];

        // if (isset($_POST) && count($_POST) > 0) {
        //     debug($_POST, false);
        // }

        if (isset($_POST['set']) && count($_POST['set']) > 0) {
            $set_condition = [];
            foreach($_POST['set'] as $selected_set) {
                if (in_array($selected_set, LorcanaBrowse::$set_list)) {
                    $set_condition[] = $selected_set;
                }
            }
            if (count($set_condition) > 0) {
                $where_condition[] = "s.`token` IN ('".implode("','", $set_condition)."')";
            }
        }

        if (isset($_POST['card_type']) && count($_POST['card_type']) > 0) {
            $card_type_condition = [];
            foreach($_POST['card_type'] as $selected_card_type) {
                if (in_array($selected_card_type, LorcanaBrowse::$card_type_list)) {
                    $card_type_condition[] = "meta.`card_type` LIKE '%".$selected_card_type."%'";
                }
            }
            if (count($card_type_condition) > 0) {
                $where_condition[] = "(" . implode(" OR ", $card_type_condition).")";
            }
        }

        if (isset($_POST['printing']) && count($_POST['printing']) > 0) {
            $card_printing_condition = [];
            foreach($_POST['printing'] as $selected_printing) {
                if (in_array($selected_printing, array_keys(LorcanaBrowse::$card_printing_list))) {
                    $card_printing_condition[] = LorcanaBrowse::$card_printing_list[$selected_printing];
                }
            }
            if (count($card_printing_condition) > 0) {
                $where_condition[] = "i.`printing` IN (".implode(",", $card_printing_condition).")";
            }
        }

        if (isset($_POST['rarity']) && count($_POST['rarity']) > 0) {
            $card_rarity_condition = [];
            foreach($_POST['rarity'] as $selected_rarity) {
                if (in_array($selected_rarity, LorcanaBrowse::$card_rarity_list)) {
                    $card_rarity_condition[] = $selected_rarity;
                }
            }
            if (count($card_rarity_condition) > 0) {
                $where_condition[] = "c.`rarity` IN ('".implode("','", $card_rarity_condition)."')";
            }
        }
        
        if (count($where_condition) > 0) {
            return 'WHERE '.implode(' AND ', $where_condition);
        }
        
        return '';

    }

    public static function user_sorting() {

        $order_condition = '';

        // if (isset($_POST) && count($_POST) > 0) {
        //     debug($_POST, false);
        // }

        if (isset($_POST['sort'])) {
            $selected_order = Common::sanitize_filter_string($_POST['sort']);
            if (in_array($selected_order, LorcanaBrowse::$order_sort_list)) {
                if ($selected_order == 'a-z') {
                    $order_condition = 'ORDER BY CONCAT(c.name, c.version)';
                } else if ($selected_order == 'best-selling') {
                    $order_condition = 'ORDER BY i.quantity ASC';
                } else if ($selected_order == 'low-to-high') {
                    $order_condition = 'ORDER BY i.price ASC';
                } else if ($selected_order == 'high-to-low') {
                    $order_condition = 'ORDER BY i.price DESC';
                }

            }
        }

        if ($order_condition != '') {
            return $order_condition;
        }
        
        return 'ORDER BY c.`set_id`, c.`iteration`';

    }

    function fetch_results() {

        $card_list = [];

        $user_filters = LorcanaBrowse::user_filters();
        $user_sorting = LorcanaBrowse::user_sorting();

        $sql = "SELECT s.*, s.name as set_name, c.*, meta.*, i.quantity as inventory_quantity, i.price as inventory_price
                FROM card c 
                    INNER JOIN `lorcana_meta` meta ON c.`id`=meta.`card_id`
                    INNER JOIN `set` s ON s.id = c.`set_id`
                    INNER JOIN `inventory` i ON i.`card_id` = c.`id`
                ".$user_filters."
                ".$user_sorting.";";

        //echo $sql . '<br/>';

        $results = $this->dbc->select($sql);

        foreach($results as $result) {

            unset($result['set_id']);
            unset($result['artist']);
            unset($result['ink_cost']);
            unset($result['card_type']);
            unset($result['classification']);
            unset($result['strength']);
            unset($result['willpower']);
            unset($result['lore_value']);
            unset($result['action_a']);
            unset($result['action_b']);

            $data = pathinfo($result['image_normal']);
            $parts = explode('?', $data['basename']);

            $result['image_normal'] = '/img/cards/lorcana/normal_'.$parts[0];

            $card_list[$result['id']] = $result;
        }

        return $card_list;

    }

    public static function sanitise_tokens() {
        
        $clean_tokens = [];

        $clean_keywords = preg_replace("/[^A-Za-z0-9 \']/", '', $_POST['keywords']);

        $tokens = explode(' ', $clean_keywords);

        foreach($tokens as $token) {

            if (strlen($token) > 2) {
                $clean_tokens[] = $token;
            }
        }

        unset($tokens);

        return $clean_tokens;

    }


}