<?php


class Musings {

  private $content = [];

  private $data_cache_file = CACHE_DIR . '/musings/contentful.json';

  function __construct() {

    $this->content = [];

  }

  private function _fetch_content() {

     // contentful.php 3.0
    $client = new Contentful\Delivery\Client(
      '3n_W-rsLqdAiBU9p3CQJTFwbj8SDM13m0GNXyjkHCI4',
      '3i2gr0wnjtvh',
      'master' // Defaults to "master" if ommitted
    );

    $renderer = new Contentful\RichText\Renderer();

    $query = new \Contentful\Delivery\Query();
    $query->setContentType('musing')->where('metadata.tags.sys.id[in]', 'lorcana')->orderBy('fields.published', true);

    $musing_data = [];

    foreach($client->getEntries($query) as $musing) {

      $hero = $musing->get('hero', null, false);
      $hero_asset = $client->getAsset($hero->getId());

      $thumb = $musing->get('thumbnail', null, false);
      $thumb_asset = $client->getAsset($thumb->getId());

      $content = $renderer->render($musing->getContent());

      $loaded_assets = $this->_load_assets($content, $client);

      if (is_array($loaded_assets)) {
        foreach($loaded_assets as $asset_key => $image_data) {
          $content = str_replace($asset_key, '<div class="'.$image_data['label'].'"><img src="'.$image_data['url'].'" /></div>', $content);
        }
      }

      //debug($musing);

      $musing_data[] = [
        'id' => $musing->getId('en-US'),
        'title' => $musing->getTitle('en-US'),
        'path' => $musing->getUrl(),
        'published' => date('F Y', strtotime($musing->getPublished())),
        'picture' => $hero_asset->getFile()->getUrl(),
        'thumbnail' => $thumb_asset->getFile()->getUrl(),
        'content' => $content,
      ];

    }

    file_put_contents($this->data_cache_file, json_encode($musing_data));

  }

  private function _load_assets($content, $client) {

    $loaded_assets = [];

    $asset_codes = preg_match_all('/<div>Asset#([\s\S]*?)<\/div>/', $content, $asset_id_list);

    if ($asset_id_list[1]) {

      for ($i=0; $i < count($asset_id_list[0]); $i++) {

        $label = '';

        $asset_data = $client->getAsset($asset_id_list[1][$i]);

        if ($asset_data->getTags()[0]) {
          $label = $asset_data->getTags()[0]->getName();
        }

        $loaded_assets[$asset_id_list[0][$i]] = [
          'url' => $asset_data->getFile()->getUrl(),
          'label' => $label,
        ];

      }

    }

    return $loaded_assets;

  }

  private function _load_from_cache() {

    //echo "Content was last modified: " . date ("F d Y H:i:s.", filemtime($this->data_cache_file));

    if (filemtime($this->data_cache_file) < strtotime('-1 second') ) {

      $this->_fetch_content();

    }

    $this->content = json_decode(file_get_contents($this->data_cache_file), true);

  }


  public function list_all() {

    $this->_load_from_cache();

    return $this->content;

  }

  public function load_musing() {


    // contentful.php 3.0
    $client = new Contentful\Delivery\Client(
      '3n_W-rsLqdAiBU9p3CQJTFwbj8SDM13m0GNXyjkHCI4',
      '3i2gr0wnjtvh',
      'master' // Defaults to "master" if ommitted
    );

    $renderer = new Contentful\RichText\Renderer();

    $query = new \Contentful\Delivery\Query();
    $query->setContentType('musing')->where('fields.url', $page_key);
    $musing_data = $client->getEntries($query)[0];


    $hero = $musing_data->get('hero', null, false);
    $asset = $client->getAsset($hero->getId());
    $musing_content = $renderer->render($musing_data->getContent());
    $published_time = strtotime($musing_data->getPublished());

    $page->content([
      'title' => $musing_data->getTitle('en-US'),
      'published_date' => date('F Y', $published_time),
      'hero_image' => $asset->getFile()->getUrl(),
      'content' => $musing_content,
    ]);


  }

}
