<?php

class ShoppingCart {

    private $dbc = null;

    private $user_id = null;

    function __construct($dbc) {
    
        $this->dbc = $dbc;

    }

    function get_cart_contents () {

        if (!isset($_SESSION['shopping_cart']) || count($_SESSION['shopping_cart']) == 0) {
            return [];
        };

        $list_of_ids = [];
        foreach($_SESSION['shopping_cart'] as $card) {
            $list_of_ids[] = Common::sanitize_filter_string($card['id'], [FILTER_FLAG_NO_ENCODE_QUOTES]);
        }

        // debug($_SESSION, false);

        $list_of_ids = implode(',', $list_of_ids);

        //debug($list_of_ids, false);

        $sql = "SELECT s.*, c.*, meta.*, i.*, c.id as id
                FROM card c 
                    INNER JOIN lorcana_meta meta ON c.id=meta.card_id
                    INNER JOIN inventory i ON c.id=i.card_id
                    INNER JOIN `set` s ON s.id = c.`set_id`
                WHERE c.id IN (".$list_of_ids.")
                ORDER BY c.name ASC";

        $results = $this->dbc->select($sql);

        $card_data_list = [];

        foreach($results as $card_data) {

            if ($card_data['card_type'] != 'Character') {
                $card_data['card_type'] = str_replace(',', ' &#9702; ', $card_data['card_type']);   
            }

            $data = pathinfo($card_data['image_normal']);
            $parts = explode('?', $data['basename']);

            $card_data['image_normal'] = '/img/cards/lorcana/normal_'.$parts[0];

            $card_data['printing'] = $card_data['printing'] == 0 ? 'normal' : 'foil';
            $card_data['quality'] = Common::card_quality($card_data['quality']);

            $card_data_list[] = $card_data;
        }

        $cards_in_cart = [];
        foreach($_SESSION['shopping_cart'] as $cart_data) {

            foreach($card_data_list as $card_data) {
                
                if ($card_data['card_id'] == $cart_data['id'] && $card_data['printing'] == $cart_data['printing']) {

                    $cards_in_cart[] = [
                        'card_data' => $card_data,
                        'cart_data' => $cart_data,
                    ];
                    continue;

                }
            }

        }

        return $cards_in_cart;

    }

    function calculate_totals($cards_in_cart) {

        $cart_totals = [
            'total_cards' => 0,
            'total_price' => 0,
        ];

        foreach($cards_in_cart as $card) {
            //debug($card);
            $cart_totals['total_cards'] += $card['cart_data']['quantity'];
            $cart_totals['total_price'] += ($card['cart_data']['quantity'] * $card['card_data']['price']);
        }

        return $cart_totals;

    }


    function add_card() {

        $found = false;

        $card_id = Common::sanitize_number($_POST['card-id']);
        $post_card_printing = Common::sanitize_filter_string($_POST['card-printing']);
        $post_card_quantity = Common::sanitize_number($_POST['card-quantity']);

        $card_printing = 0;
        if($post_card_printing == 'foil') {
            $card_printing = 1;
        }

        $sql = "SELECT i.quantity FROM inventory i WHERE i.`card_id`=".$card_id." AND `printing`=".$card_printing.";";

        $inventory_data = $this->dbc->select($sql, true);

        for($i = 0; $i < count($_SESSION['shopping_cart']); $i++) {
            if ($_SESSION['shopping_cart'][$i]['id'] == $card_id) {
                //echo 'is '.$inventory_data['quantity'].' less than or equal to '.($_SESSION['shopping_cart'][$i]['quantity'] + 1).'</br>';
                if ($inventory_data['quantity'] >= $_SESSION['shopping_cart'][$i]['quantity'] + 1) {
                    $_SESSION['shopping_cart'][$i]['quantity']++;    
                }
                $found = true;                
                break;
            }
        }

        if ($found == false) {
            $_SESSION['shopping_cart'][] = [
                'id' => $_POST['card-id'],
                'printing' => $_POST['card-printing'],
                'quantity' => ($inventory_data['quantity'] >= $post_card_quantity) ? $post_card_quantity : $inventory_data['quantity'],
            ];
        }
        
    }

    function update_card() {

        $updated = false;

        $card_id = Common::sanitize_number($_POST['card-id']);
        $post_card_printing = Common::sanitize_filter_string($_POST['card-printing']);
        $post_card_quantity = Common::sanitize_number($_POST['card-quantity']);

        $card_printing = 0;
        if($post_card_printing == 'foil') {
            $card_printing = 1;
        }

        $sql = "SELECT i.quantity FROM inventory i WHERE i.`card_id`=".$card_id." AND `printing`=".$card_printing.";";

        $inventory_data = $this->dbc->select($sql, true);

        for($i = 0; $i < count($_SESSION['shopping_cart']); $i++) {
            if ($_SESSION['shopping_cart'][$i]['id'] ==  $card_id) {
                if ($inventory_data['quantity'] >= $post_card_quantity) {
                    $_SESSION['shopping_cart'][$i]['quantity'] = $post_card_quantity;    
                }
                else {
                    $_SESSION['shopping_cart'][$i]['quantity'] = $inventory_data['quantity'];    
                }
                $updated = true;
                break;
            }
        }

        header('Content-Type: application/json; charset=utf-8');
        
        if ($updated) {

            echo json_encode([ 
                'action' => 'update', 
                'status' => 'success',
            ]); 

        } else {
            
            echo json_encode([ 
                'action' => 'update', 
                'status' => 'failed',
            ]); 

        }

        exit; // Nowa feyah!
        
    }

    function remove_card() {

        $removed = false;

        $card_id = Common::sanitize_number($_POST['card-id']);
        $post_card_printing = Common::sanitize_filter_string($_POST['card-printing']);

        for($i = 0; $i < count($_SESSION['shopping_cart']); $i++) {
            if ($_SESSION['shopping_cart'][$i]['id'] == $card_id && $_SESSION['shopping_cart'][$i]['printing'] == $post_card_printing) {
                unset($_SESSION['shopping_cart'][$i]);            
                $removed = true; 
                break;
            }
        }

        header('Content-Type: application/json; charset=utf-8');
        
        if ($removed) {

            // reindex the array
            $_SESSION['shopping_cart'] = array_values($_SESSION['shopping_cart']);

            echo json_encode([ 
                'action' => 'remove', 
                'status' => 'success',
            ]); 

        } else {
            
            echo json_encode([ 
                'action' => 'remove', 
                'status' => 'failed',
            ]); 

        }

        exit; // Noa feah!

    }

    public static function items_in_cart () {

        $count = 0; 
        
        foreach($_SESSION['shopping_cart'] as $item) { 
            $count += $item['quantity'];
        }

        return $count;
    }

    public static function postage_options () {

        return [
            [
                'key' => 'SCL',
                'label' => '2nd Class Letter',
                'times' => '2-3 working days',
                'tracking' => 'Tracking not included',
                'price' => 0.85,
            ],
            [
                'key' => 'FCL',
                'label' => '1st Class Letter',
                'times' => '1 working days',
                'tracking' => 'Tracking not included',
                'price' => 1.65,
            ],
            [
                'key' => 'SPD',
                'label' => 'Special Delivery',
                'times' => '1 working day',
                'tracking' => 'Tracking included',
                'price' => 8.35,
            ]
        ];


    }

    

}