<?php

// Start our sessions
session_name('_tcgfriends');
session_start();
$_SESSION['id'] = session_id();
if(!isset($_SESSION['authenticated'])) {
	$_SESSION['authenticated'] = false;
}

// mute dump regex
ini_set("pcre.jit", "0");
ini_set("display_errors", 1);
ini_set('memory_limit','32M');

// Set our error reporting levels
error_reporting(E_ALL ^ E_DEPRECATED);

$_SESSION['queries'] = [];

if(!isset($_SESSION['shopping_cart'])) {
	$_SESSION['shopping_cart'] = [];
	$_SESSION['collection'] = [];
}

function debug($data, $die=true) {
	echo 'debug data:: <xmp>'; print_r($data); echo '</xmp>';
	if ($die) {
		exit;
	}
}
function dts($label, $string) {
	echo $label . ': '.$string.'<br/>';
}
function convert($size) {
	$unit=array('b','kb','mb','gb','tb','pb');
	return @round($size/pow(1024,($i=floor(log($size,1024)))),2).' '.$unit[$i];
}

include_once 'environment.prod.php';

// Global settings
define('APP_NAME', 'TCG Friends');
define('TEMPLATE_DIR', DOCROOT . 'templates');
define('CONTROLLER_DIR', DOCROOT . 'controllers');
define('CLASS_DIR', DOCROOT . 'app/classes/');
define('LIB_DIR', DOCROOT . 'app/library/');
define('TMP_DIR', DOCROOT . 'tmp/');
define('CACHE_DIR', DOCROOT . 'cache');
define('SITE_ROOT', '/');

require_once DOCROOT.'vendor/autoload.php';

// Create our Page object
include_once 'class.DB.php';
include_once 'class.Page.php';
include_once CLASS_DIR . 'class.Common.php';
include_once CLASS_DIR . 'class.ShoppingCart.php'; 

$dbc = new Database();

$page = new Page($dbc);

$set_auth_list = [
	'/account',
	'/login',
	'/admin',
];

$page->set_auth_list($set_auth_list);

$page->defined_routes([
	'card/$' => 'page.card',
	'musing/$' => 'page.musing',
]);

// Menu (Primary)
$menu = [
	[
		'title' => 'Browse Singles',
		'link' => '/browse',
		'active' => false
	],
	// [
	// 	'title' => 'Complete my set',
	// 	'link' => '/complete-my-set',
	// 	'active' => false
	// ],
	[
		'title' => 'Musings',
		'link' => '/musings',
		'active' => false
	],
	// [
	// 	'title' => 'Podcast',
	// 	'link' => '/podcast',
	// 	'active' => false
	// ],
	[
		'title' => 'About',
		'link' => '/about',
		'active' => false
	],
];

$page->set_active_menu($menu);

//debug($menu);

$page->set_menu('primary_menu', $menu);

// Initialize the page we have based on set config
$page->init();

$page->content(array(
	'shopping_cart_items' => ShoppingCart::items_in_cart(),
	'value_spotlight' => Common::highlight_value(),
	'server' => gethostname(),
	'current_year' => Date('Y'),
));

// Load the controller
if ($controller = $page->getController()) {
	include_once($controller);
}
