<?php

/**
 * Database wrapper for a MySQL
 */

class Database {

	// The database connection
	protected static $connection;

	/**
	 * Connect to the database
	 *
	 * @return bool false on failure / mysqli MySQLi object instance on success
	 */
	public function connect() {

		// Try and connect to the database
		if(!isset(self::$connection)) {
			// Load configuration as an array. Use the actual location of your configuration file
			// Put the configuration file outside of the document root
			self::$connection = new mysqli("localhost", "oscar", "&XlkV>64yig%:]P_", "tcgfriends");

			self::$connection->set_charset("utf8mb4");
		}

		// If connection was not successful, handle the error
		if(self::$connection === false) {
			// Handle error - notify administrator, log to a file, show an error screen, etc.
			return false;
		}

		return self::$connection;
	}

	/**
	 * Query the database
	 *
	 * @param $query The query string
	 * @return mixed The result of the mysqli::query() function
	 */
	public function query($query) {
		// Connect to the database
		$connection = $this->connect();

		$_SESSION['queries'][] = $query;

		// Query the database
		$result = $connection->query($query);

		return $result;
	}

	/**
	 * Insert row into the database
	 *
	 * @param $table The table for insertion
	 * @param $values The columns and values to insert
	 * @return mixed The result of the mysqli::query() function
	 */
	public function insert($table, $values) {
		// Connect to the database
		$connection = $this->connect();

		// Build the query
		$query = "INSERT INTO ".$table." (`".implode("`,`", array_keys($values))."`) VALUES ('".implode("','", array_values($values))."')";

		// Query the database
		$result = $connection->query($query);

		if(self::$connection->error) {
			throw new Exception(self::$connection->error . '  in "' . $query . '"');
		}

		return $result;
	}

	/**
	 * Fetch rows from the database (SELECT query)
	 *
	 * @param $query The query string
	 * @return bool False on failure / array Database rows on success
	 */
	public function select($query, $single=false) {

		$rows = array();
		$result = $this->query($query);

		if($result === false) {

			if(self::$connection->error) {
				throw new Exception(self::$connection->error . '  in "' . $query . '"');
			}

			return false;

		}

		while ($row = $result->fetch_assoc()) {
			$rows[] = $row;
		}

		if ($single && count($rows) == 1) {
			return $rows[0];
		}

		return $rows;

	}

	/**
	 * Fetch the last error from the database
	 *
	 * @return string Database error message
	 */
	public function error() {
		$connection = $this->connect();
		return $connection->error;
	}

	/**
	 * Quote and escape value for use in a database query
	 *
	 * @param string $value The value to be quoted and escaped
	 * @return string The quoted and escaped string
	 */
	public function sanitise($value) {
		$connection = $this->connect();
		return $connection->real_escape_string($value);
	}

}
