<?php

require_once 'class.Component.php';

class Page extends Component {

	private $querystring = '';

	private $breadcrumb = '';

	private $defined_routes = [];

	private $authentication_list = [];

	public function __construct() {
		parent::__construct();
  }

  public function __destruct() {
    parent::__destruct();
  }

	public function init() {
		$this->router();
	}

  public function defined_routes($routes) {
    $this->defined_routes = $routes;
  }

  public function get_querystring() {
    $querystring = array();
    if ($this->querystring) {
	    parse_str($this->querystring, $querystring);
    }
    return $querystring;

  }

    private function router() {

		$path = parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH);
		$this->querystring = parse_url($_SERVER['REQUEST_URI'], PHP_URL_QUERY);

		// echo 'SESSION: <xmp>'; print_r($_SESSION); echo '</xmp>';
		//
		// echo 'Path: ' . $path . '<br/>';
		// echo 'Querystring: ' . $querystring . '<br/>';
		// //exit;

		// take the path from the URL sanitize it and find the appropriate controller to load along with the template.

		$path = rtrim($path,'/');

		if ('' == $path) {
			$clean_path = '/home';
		} else {
			$clean_path = htmlspecialchars($path, ENT_QUOTES, 'UTF-8');
		}

		// Process our authentication here.
		$access_ok = true;
		foreach($this->authentication_list as $exception) {
			if ($exception == substr($clean_path, 0, strlen($exception))) {
				$access_ok = false;
				break;
			}
		}

		if ($access_ok == false && $_SESSION['authenticated'] == false) {
			$this->setController(null);
			$this->setTemplate('page.401.twig');
			return;
		}

		if ($route_info = $this->is_defined_route($clean_path)) {
			global $page_key;
			$page_key = $route_info['key'];
			$controller = DOCROOT.'controllers/'.$route_info['route'].'.php';
			$template = $route_info['route'].'.twig';
		} else {
			$controller = DOCROOT.'controllers/page'.str_replace('/', '.', $clean_path).'.php';
			$template = 'page'.str_replace('/', '.', $clean_path).'.twig';
		}

		// echo 'Path: ' . $clean_path . '<br/>';
		// echo 'Controller: ' . $controller . '<br/>';
		// echo 'Template: ' . $template . '<br/>';
		//
		// echo '-------------------<br/>';

		if (!file_exists($controller)) {
			$controller = null;
			$template = 'page.404.twig';
		}

		// echo 'Path: ' . $clean_path . '<br/>';
		// echo 'Controller: ' . $controller . '<br/>';
		// echo 'Template: ' . $template . '<br/>';
		// exit;

		$this->setController($controller);
		$this->setTemplate($template);

	}

    private function is_defined_route($clean_path) {

	    //echo 'clean path: ' . $clean_path. '<br/>';
	    //echo 'defined routes <xmp>'; print_r($this->defined_routes); echo '</xmp>';

	    // our path becomes the test material
	    $path_parts = explode('/', ltrim($clean_path,'/'), 4);
		$path_number_parts = count($path_parts);

	    //echo 'path_parts: <xmp>'; print_r($path_parts); echo '</xmp>';
	    //echo 'path number parts: ' . $path_number_parts. '<br/>';
		//exit;

		foreach($this->defined_routes as $defined_route => $route_path) {

			$match = false;

			$return = array();

			//echo '$defined_route: ' . $defined_route. '<br/>';
			//echo '$path: ' . $route_path. '<br/>';

			// our route needs to be taken apart for analysis
	    	$route_parts = explode('/', $defined_route, 4);
			$route_number_parts = count($route_parts);

		    //echo 'routeparts: <xmp>'; print_r($route_parts); echo '</xmp>';
		    //echo 'route number parts: ' . $route_number_parts. '<br/>';

			//echo 'path parts: <xmp>'; print_r($path_parts); echo '</xmp>';
		    //echo 'path number parts: ' . $path_number_parts. '<br/>';
			//exit;

			for ($i=0; $i<$path_number_parts; $i++) {

				if (!isset($path_parts[$i]) || !isset($route_parts[$i])) {
					//echo 'PATH PARTS::<xmp>'; print_r($path_parts); echo '</xmp>';
					//echo 'ROUTE PARTS::<xmp>'; print_r($route_parts); echo '</xmp>';
					//echo 'FAIL - There are not '.$i.' elements to test<br/>';
					$match = false;
					break;
				}

				//echo $i.' :: testing if '.$path_parts[$i].' == '.$route_parts[$i].'<br/>';

				if ($path_parts[$i] == $route_parts[$i]) {
					//$i.' :: MATCHED! ('.$path_parts[$i].' & '.$route_parts[$i].')<br/>';
					$match = true;
					continue;
				} else if (('%' == $route_parts[$i]) && is_numeric($path_parts[$i])) {
					$return = array(
						'route' => $route_path,
						'key' => $path_parts[$i],
					);
					//echo $i.' :: MATCHED token :: '.$route_path;
					$match = true;
					continue;
				} else if (('$' == $route_parts[$i])) {
					$return = array(
						'route' => $route_path,
						'key' => Common::check_plain($path_parts[$i]),
					);
					//echo $i.' :: MATCHED token :: '.$route_path;
					$match = true;
					continue;
				} else {
					$match = false;
				}

				if(!$match) {
					break;
				}

			}

			//echo 'END :: did we find a match '.($match ? 'yes' : 'no').'<br/>';

			if ($match) {
				$this->breadcrumb = $path_parts;
				return $return;
			}
			//$i.' :: UGH, SIGH, NO MATCH onto the next ROUTE!<hr/>';

		}

    }

    public function set_menu($name, $menu) {
	    $this->content(array($name => $menu));
    }

	public function set_auth_list($list) {
		$this->authentication_list = $list;
	}

    public function set_active_menu(&$menu) {

	    $path = parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH);
	    $clean_path = htmlspecialchars(rtrim($path,'/'), ENT_QUOTES, 'UTF-8');
	    $breadcrumb = array_values(array_filter(explode('/', $clean_path)));

	    if (!count($breadcrumb)) {
		    return;
	    }

	    foreach($menu as &$item) {

		    //echo 'TEST :: '.ltrim($item['link'], '/').' = '.$breadcrumb[0].'<br/>';

			if (ltrim($item['link'],'/') == $breadcrumb[0]) {

				$item['active'] = 'true';

				if (isset($item['children'])) {

					foreach($item['children'] as &$subitem) {

						$path = implode('/', array_slice($breadcrumb, 0, 2));

						//echo 'SUB TEST :: '.ltrim($subitem['link'],'/').' = '.$path.'<br/>';

						if (isset($subitem['link']) && ltrim($subitem['link'],'/') == $path) {

							//echo 'ACTIVE :: '.ltrim($subitem['link'],'/').'<br/>';

							$subitem['active'] = 'true';

							break;

						}

					}

				}

		   }

	    }

	    /*
	    debug($clean_path, false);
	    debug($breadcrumb, false);
	    debug($menu);
	    */

    }


}
